/*
 * Copyright (c) 2001 Leon Woestenberg <leon.woestenberg@axon.nl>
 * Copyright (c) 2001 Axon Digital Design B.V., The Netherlands.
 * All rights reserved.
 *
 * This is a device driver for the Crystal Semiconductor CS8900
 * chip in combination with the lwIP stack.
 *
 * This is work under development. Please coordinate changes
 * and requests with Leon Woestenberg <leon.woestenberg@axon.nl>
 *
 * The Swedish Institute of Computer Science and Adam Dunkels
 * are specifically granted permission to redistribute this
 * source code under any conditions they seem fit.
 *
 * A quick function roadmap:
 *
 * cs8900_*() are low level, cs8900 hardware specific functions.
 * These are static functions of the device driver source and
 * SHOULD NOT need to be called from outside this source.
 *
 * cs8900if_*() are the lwIP network interface functions.
 * 
 * cs8900_interrupt() is an early interrupt service routine (ISR).
 * It merely sets a flag to indicate the cs8900 needs servicing.
 * (This function MAY be tied to an interrupt vector, IF present).
 *
 * cs8900_service() is the actual interrupt event service routine.
 * It must be called whenever the cs8900 needs servicing. It MAY
 * be polled safely (so, you do NOT NEED interrupt support.)
 *
 * cs8900_init() sets up the cs8900, using its register set. When
 * using the driver on your particular hardware platform, make sure
 * the register setups match.
 * Function is called from cs8900if_init().
 *
 * cs8900_input() transfers a received packet from the chip.
 * Function is called from cs8900if_input().
 *
 * cs8900_output() transfers a packet to the chip for transmission.
 * Function is called from cs8900if_output().
 *
 * cs8900if_init() initializes the lwIP network interface, and
 * calls cs8900_init() to intialize the hardware.
 * Function is called from lwIP.
 * 
 * cs8900if_service() is the service routine, which must be called
 * upon the need for service, or on a regular basis, in order to
 * service the Ethernet chip.
 *
 * cs8900if_input() calls cs8900_input() to get a received packet
 * and then forwards the packet to protocol(s) handler(s).
 * Function is called from cs8900_service().
 *
 * cs8900if_output() resolves the hardware address, then
 * calls cs8900_output() to transfer the packet.
 * Function is called from lwIP.
 *
 * Future development:
 * 
 * Split the generic Ethernet functionality (a lot of the
 * cs8900if_*() functions) and the actual cs8900a dependencies.
 *
 * Enhance the interrupt handler to service the Ethernet
 * chip (to decrease latency); support early packet
 * inspection (during reception) to early drop unwanted
 * packets, minimize chip buffer use and maximize throughput.
 *
 * Statistics gathering.
 *
 */
#include "general.h"
#include "lwip/debug.h"

#include "lwip/opt.h"
#include "lwip/def.h"
/*
#include "lwip/mem.h"
#include "lwip/pbuf.h"
#include "lwip/stats.h"
#include "lwip/sys.h"
#include "netif/arp.h" 
*/

//#include "gui.h"  //No hay funciones de display ahora

#include <string.h>//Para poder utilizar la funcin malloc
#include "hardware.h"//Para inicializar el CS8900A para el hardware de nuestra placa
/*
#ifdef SFR18 // Driver implementor's hardware 
#  include "leds.h"
#else
#  define leds_on()
#  define leds_off()
#endif
*/
#include "cs8900if.h"
#define	Eth_ARP		1	//Se define las funciones externas de los mdulos q se necesitan
#define	Eth_PBUF	1
#define Eth_MEM		1
#define Eth_IP_ADDR	1		

#include "eth_lwip.h" //para funciones de otros mdulos, ahora no hace falta


#if defined(CS8900_TX_QUEUE) && (CS8900_TX_QUEUE > 0)
#  include "lwip/ip.h"
#  include "lwip/ip_addr.h"
#endif

#pragma memory=default
/* Define those to better describe your network interface. */
#define IFNAME0 'e'
#define IFNAME1 'n'

static const struct  eth_addr ethbroadcast = {{0xffU,0xffU,0xffU,0xffU,0xffU,0xffU}};

/* Forward declarations. */
static void  cs8900if_input(struct netif *netif); //Natalia
static err_t cs8900if_output(struct netif *netif, struct pbuf *p, //Natalia
			       struct ip_addr *ipaddr);                //Natalia

static err_t cs8900_output(struct netif *netif, struct pbuf *p);   //Natalia
//static struct pbuf *cs8900_input(struct netif *netif,u16_t event_type);         //Natalia
struct pbuf *cs8900_input(struct netif *netif);//,u16_t event_type);         //Natalia

static void cs8900_service(struct netif *netif);                //Natalia
static u32_t cs8900_chksum(void *dataptr, int len);
void cs8900_init(struct netif *netif);
/* Define these to match your hardware setup */  //Natalia 
#define MEM_BASE 	0xA00000		//Natalia0xE000
#define IO_BASE 	0x0300			//Natalia0x0800
#define INT_NR 		0x00			//Natalia0x00

#pragma memory=huge

unsigned int *pPacketPageTxCMD;//#define TXCMD    *(volatile u16_t *)(MEM_BASE + IO_BASE + 0x04)              //Transmit in interrupt mode
unsigned int *pPacketPageLength;//#define TXLENGTH *(volatile u16_t *)(MEM_BASE + IO_BASE + 0x06)
unsigned int *pPacketPageISQ;//#define ISQ      *(volatile u16_t *)(MEM_BASE + IO_BASE + 0x08)
unsigned int *pPacketPagePointer;	//#define PACKETPP *(volatile u16_t *)(MEM_BASE + IO_BASE + 0x0A)
unsigned int *pPacketPageData; //#define PPDATA   *(volatile u16_t *)(MEM_BASE + IO_BASE + 0x0C)
unsigned int *pPacketPageRxTxData;//#define RXTXREG  *(volatile u16_t *)(MEM_BASE + IO_BASE)

#pragma memory=default
#define nMAXEth				0x400	//=1024, Max num datos, incluidas cabeceras en una trama de ethernet
unsigned int uiEthBuffer_Rx1[nMAXEth];   	//Buffer de datos recibidos por ethernet

#define TXCMD		(MEM_BASE + IO_BASE + 0x04)
#define TXLENGTH	(MEM_BASE + IO_BASE + 0x06)
#define ISQ 		(MEM_BASE + IO_BASE + 0x08)
#define PACKETPP 	(MEM_BASE + IO_BASE + 0x0A)
#define PPDATA		(MEM_BASE + IO_BASE + 0x0C)
#define RXTXREG		(MEM_BASE + IO_BASE)
// CS8900 PacketPage register offsets 

#define  CS_PP_EISA        0x0000          // EISA Registration number of CS8900
#define  CS_PP_PRODID      0x0002          // Product ID Number
#define  CS_PP_IOBASE      0x0020          // I/O Base Address
#define  CS_PP_INTNUM      0x0022          // Interrupt number (0,1,2, or 3)
#define  CS_PP_RXCFG       0x0102          // Receiver Configuration
#define  CS_PP_RXCTL       0x0104          // Receiver Control
#define  CS_PP_TXCFG       0x0106          // Transmit Configuration
#define  CS_PP_BUFCFG      0x010A          // Buffer Configuration
#define  CS_PP_LINECTL     0x0112          // Line Control Register offset
#define  CS_PP_SELFCTL     0x0114          // Self Control
#define  CS_PP_BUSCTL      0x0116          // Bus Control
#define  CS_PP_TESTCTL     0x0118          // Test Control
#define  CS_PP_ISQ         0x0120          // Interrupt status queue
#define  CS_PP_RXEVENT     0x0124          // Receiver Event
#define  CS_PP_TX_EVENT    0x0128          // Transmitter Event
#define  CS_PP_BUF_EVENT   0x012C          // Buffer Event
#define  CS_PP_RXMISS      0x0130          // Receiver Miss Counter
#define  CS_PP_TXCOL       0x0132          // Transmit Collision Counter
#define  CS_PP_LINESTATUS  0x0134          // Line Status
#define  CS_PP_SELFTEST    0x0136          // Self Status
#define  CS_PP_BUSSTATUS   0x0138          // Bus Status
#define  CS_PP_TXCMD       0x0144          // Transmit Command Request
#define  CS_PP_TXLEN       0x0146          // Transmit Length
#define  CS_PP_IA1         0x0158          // Individual Address (IA)
#define  CS_PP_IA2         0x015A          // Individual Address (IA)
#define  CS_PP_IA3         0x015C          // Individual Address (IA)

#define  CS_PP_RXSTATUS    0x0400          // Receive Status
#define  CS_PP_RXLEN       0x0402          // Receive Length
#define  CS_PP_RXFRAME     0x0404          // Receive Frame Location
#define  CS_PP_TXFRAME     0x0A00          // Transmit Frame Location

#define ETHADDR0 (0x00U)
#define ETHADDR1 (0x03U)
#define ETHADDR2 (0x41U)
#define ETHADDR3 (0xbbU)
#define ETHADDR4 (0xaaU)
#define ETHADDR5 (0xddU)

//static struct netif *cs8900if_netif;      //Natalia
//struct netif *cs8900if_netif;      //Natalia
unsigned char ucNada;

void cs8900_nada(void)
{
	ucNada=0;
}

void cs8900_iniVariables(void)
{
	pPacketPageTxCMD=(unsigned int *)TXCMD;
	pPacketPageLength=(unsigned int *)TXLENGTH;
	pPacketPageISQ=(unsigned int*)ISQ;
	pPacketPagePointer=(unsigned int *)PACKETPP;
	pPacketPageData=(unsigned int *)PPDATA;
	pPacketPageRxTxData=(unsigned int*)RXTXREG;
}

// hardware interrupt vector handler 
interrupt [0x40] void cs8900_interrupt(void)//Natalia, el valor del vector interrupt=0x40 en lugar de 0x18
{
  struct cs8900if *cs8900if;// = cs8900if_netif->state;
  if (cs8900if)
  {
    cs8900if->needs_service = 1;
#if (CS8900_STATS > 0)
    cs8900if->interrupts++;
#endif
  }
  //leds_on(LED_FP0);//Natalia
  
  
 // GUI_DispStringAt("UDP OK",0,0);
}

// cs8900_init()
//
// initializes the CS8900A
void cs8900_init(struct netif *netif)         //Natalia
{
  u16_t dummy;
 

  Ethernet_AEN=0; //Como output, para el CS8900A
  cs8900_iniVariables(); //Necesario para poder inicializar el chip en I/O Mode
  //leds_off(LED_FP0);//Natalia

  /* Obtain MAC address from network interface. */

  /* XXX: make up an address. */ //Natalia ???
  ((struct cs8900if *)(netif->state))->ethaddr->addr[0] = ETHADDR0;
  ((struct cs8900if *)(netif->state))->ethaddr->addr[1] = ETHADDR1;
  ((struct cs8900if *)(netif->state))->ethaddr->addr[2] = ETHADDR2;
  ((struct cs8900if *)(netif->state))->ethaddr->addr[3] = ETHADDR3;
  ((struct cs8900if *)(netif->state))->ethaddr->addr[4] = ETHADDR4;
  ((struct cs8900if *)(netif->state))->ethaddr->addr[5] = ETHADDR5;
   
  // set RESET bit                //----------------Nat--------------------
  *pPacketPagePointer = CS_PP_SELFCTL;       //Acceso al registro SELF_ctl, register 15
  *pPacketPageData = 0x0055U;               //Valores por defecto mas el bit de Reset a 1.
  								  //Esto origina el reset del chip, y cuando ha finalizado
  								  //el bit se pone a cero                   
  dummy=*pPacketPageData;
                                  //-------------fin Nat--------------------
  // { the RESET bit will be cleared by the cs8900a
  //   as a result of the reset }
                                  //----------------Nat--------------------
                                  //Ahora se espera a q el bit est a cero
                                  //reset finalizado
                                  //-------------fin Nat--------------------
  // RESET bit cleared?
  while((*pPacketPageData & 0x0040U) != 0); // TODO: add timeout

  // { after full initialization of the cs8900a
  //   the INITD bit will be set }
  //Aadido por Natalia
  *pPacketPagePointer =CS_PP_IOBASE;
   if(*pPacketPageData!=0x300){return;} //Error inicializando el CS8900a
                                  //----------------Nat--------------------
  *pPacketPagePointer = CS_PP_SELFTEST;      //Self status, register 16
  								  //Mira q el bit InitD(bit8) est a uno, es decir
  								  //inicializacin del chip completa        
  								  //-------------fin Nat--------------------
  								  
  // INITD bit still clear?
  while ((*pPacketPageData & 0x0080U) == 0); // TODO: add timeout
  // { INITD bit is set }
                                  //----------------Nat--------------------      
                                  //Un par de lecturas porque los dos primeros accesos 
                                  //pueden funcionar mal, para q se pueda leer luego
                                  //-------------fin Nat--------------------
  // TODO: check if this is correct and/or necessary 
    *pPacketPagePointer = CS_PP_SELFTEST;      //Self status, register 16
	dummy =*pPacketPageData;
//  dummy = *(u8_t *)(MEM_BASE + IO_BASE + 0x0D) ;
  /* Dummy read, put chip in 16bit-mode */
    *pPacketPagePointer = CS_PP_SELFTEST;      //Self status, register 16
	dummy=*pPacketPageData;
//  dummy = *(u16_t *)(MEM_BASE + IO_BASE + 0x0D) ; 

  // Set MAC address              //----------------Nat--------------------      
                                  //IEEE 48bit-Individual Address (pag 86, Apartado3.3, 
                                  // del CS8900A Datasheet
                                  //-------------fin Nat--------------------
  *pPacketPagePointer = CS_PP_IA1;
  *pPacketPageData = (u16_t)ETHADDR0 | ((u16_t)ETHADDR1 << 8U);
  *pPacketPagePointer = CS_PP_IA2;
  *pPacketPageData = (u16_t)ETHADDR2 | ((u16_t)ETHADDR3 << 8U);
  *pPacketPagePointer = CS_PP_IA3;
  *pPacketPageData = (u16_t)ETHADDR4 | ((u16_t)ETHADDR5 << 8U);
                                  //----------------Nat--------------------    
                                  //Configuar el acceso de tramas, unicast, broadcast, multicast..
                                  //Rx_ctl, register 5
                                  //0000 1101 00 000101
                                  //	 1------------->broadcast
                                  //      1------------>unicast
                                  //		1---------->acepta tramas de 64<longitud<1518bytes y CRC vlido
                                  //			 000101 valores fijos
                                  //-------------fin Nat--------------------
  
  //Physical Interface Selection
  *pPacketPagePointer=CS_PP_LINECTL; //10BaseT, transmision y recepcipn habilitadas
  dummy=*pPacketPageData;
  dummy|=0x00D3;             		
  *pPacketPageData=dummy;


  // accept valid unicast or broadcast frames
  *pPacketPagePointer = CS_PP_RXCTL;
  *pPacketPageData = 0x0D05; //(0x0005U | 0x0800U/*broadcast*/ | 0x0400U/*individual*/ | 0x0100U/*RxOK*/);
                                  //----------------Nat--------------------    
                                  //Rx_cfg, register 3, receiver configuration
                                  //0000 0001 00 000011
                                  //		1---------->RxiOK, interrupt at reception OK
                                  //			 000011 fijo
                                  //-------------fin Nat--------------------
  // enable receive interrupt
  *pPacketPagePointer = CS_PP_RXCFG;
  *pPacketPageData = (0x0003U | 0x0100U/*RXIRQ*/);
                                  //----------------Nat--------------------      
                                  //Tx,_cfg, register 7, Transamit configuration
                                  //0000 0000 0000 0111
                                  //Int por transmisin completa inhabilitada
                                  //-------------fin Nat--------------------
  // disable transmit interrupt (is default)
  *pPacketPagePointer = CS_PP_TXCFG;
  *pPacketPageData = 0x0103;//(0x0007U | 0);
                                  //----------------Nat--------------------    
                                  //Int number,el q hemos puesto, por defecto 4
                                  //en nuestro caso 0
                                  //-------------fin Nat--------------------
  // use interrupt number 0
  *pPacketPagePointer = CS_PP_INTNUM;
  *pPacketPageData = (0x0000U);
                                  //----------------Nat--------------------    
                                  //Buf_cfg, register B, Buffer configuration
                                  //CS8900_STATS=2, tratamiento de dos paquetes
                                  //0011 0100 00 001011   
                                  //  1---------------->RsMiss counter.Int, when a packet is lost due to slow movement of receive data in buffers
                                  //   1--------------->cuando RxD+/RxD- activos cuando pacquete se est transmitiendo
                                  //	  1------------>RxMiss,Int, when lost packet received due to slow movemnt
                                  //			001011->fijos
                                  //-------------fin Nat--------------------

  *pPacketPagePointer=CS_PP_TESTCTL;
  *pPacketPageData =0x0080;		//Para debug, para poder enviar y recibir independientemente de link pulses


 
  // generate interrupt event on:
  // - the rxMISS counter reaches 0x200, or
  // - a received frame is lost
  *pPacketPagePointer = CS_PP_BUFCFG;
  *pPacketPageData = (0x000bU |
#if (CS8900_STATS > 0) // interrupt before counter overflow 
  (0x2000U/*MissOvfloiE*/ | 0x1000U/*TxColOvfloiE*/) |
#endif
#if (CS8900_STATS > 1) // interrupt on counter increment
  (0x0400U/*RxMissiE*/) |
#endif
  0x0000);
                                  //----------------Nat--------------------    
                                  //Register 17, bus Control
                                  //1000 0000 00 010111
                                  //1------------------>Cs8900A generate interrupts
                                  //			 010111->fijos
                                  //-------------fin Nat--------------------
  // enable interrupt generation
  *pPacketPagePointer = CS_PP_BUSCTL;
  //*pPacketPageData = (0x0017U | 0x8000U/*EnableIRQ*/);
  *pPacketPageData=0x8417; 		//Enable IRQs, cs8900a can operate in Memory Mode
  // enable:
  // - receiver
  // - transmitter                 
                     				//----------------Nat--------------------  
                     				//Register 13, Line_ctl
                     				//0000 0000 11	010011
                     				//			1---------> TxOn
                     				//			 1--------> RxOn
                     				//				010011->fijos
                     				//-------------fin Nat--------------------   
  *pPacketPagePointer = CS_PP_LINECTL;
  *pPacketPageData = (0x0013U | 0x0080U/*SerTxOn*/ | 0x0040U/*SerRxOn*/);
}

static err_t cs8900_output(struct netif *netif, struct pbuf *p)   //Natalia
{
    struct pbuf *q;        //Natalia
	int tries = 0;             
	                                          //----------------Nat--------------------   
	                                          // LineST, Line status, register 14
	                                          // Mira el bit LinkOK=1, entonces el receptor ha detectado pulsos, link OK
	                                          // En caso contrario no ha habido actividad durante al menos 50ms
                                              //-------------fin Nat--------------------  
  // exit if link has failed
  *pPacketPagePointer = CS_PP_LINESTATUS;
  if ((*pPacketPageData & 0x0080U/*LinkOK*/) == 0) return ERR_CONN; // no Ethernet link      
  											  //----------------Nat--------------------   
  											  //Transmit in I/O mode, pag 98, CS8900A Datasheet
  											  //TXCMD register access in I/O Mode= IOBase+0x0004
  											  //0000 0000 11	001001
  											  //		  1------------>Transmision will be terminated after only one colision
  											  //		   1----------->Force: When transmiting a new frame, any waiting frame in the buffer is deleted
  											  //							  the frame being sent is terminated with a bad CRC
  											  //				001001->fijos
                                              //-------------fin Nat--------------------  
  /* transmit command */
  *pPacketPageTxCMD = 0x00C9U;
  *pPacketPageLength = p->tot_len;
                                             //----------------Nat-------------------- 
                                             //Register 18, Bus status
                                             //Mientras el bit8, Rdy4Tx NOW=0, el chip no est preparado para transmisin
                                             //Se toca el registro RxCFG, register 3, para ver si entonces el chip
                                             //se prepara para transmisin                                    
                                             //0000 0001 01 000011
                                             //		   1----------->RxOKiE,interrupt si ha llegado una trama OK
                                             //			  1-------->Skip_1, descarta la trama q se est recibiendo
                                             //				000011->fijos
                                             //-------------fin Nat--------------------  
  *pPacketPagePointer = CS_PP_BUSSTATUS;             	  
  // not ready for transmission and still within 100 retries?   
  
  while(((*pPacketPageData & 0x0100U/*Rdy4TxNOW*/) == 0) && (tries++ < 100))
  {
    // throw away the last committed received frame
    *pPacketPagePointer = CS_PP_RXCFG;
    *pPacketPageData = (0x0003U | 0x0040U/*Skip_1*/ | 0x0100U/*RxOKiE*/);
    *pPacketPagePointer = CS_PP_BUSSTATUS;
    /* cs8900if->dropped++; CHECK: we do not know if we actually will drop a frame here */ 
  }
  // ready to transmit?                             
  											 //----------------Nat-------------------- 
                                             //Register 18, Bus status
                                             //Mira si el bit8, Rdy4Tx NOW=1,es decir si el chip est preparado
                                             //para aceptar una trama para enviar.
                                             //-------------fin Nat-------------------
  if((*pPacketPageData & 0x0100U/*Rdy4TxNOW*/) != 0)    //----------------Nat--------------------
  {                                           //Se van enviando los bytes
 //   pbuf *q;        //Natalia               //Siempre q el chip lo permita
    for(q = p; q != NULL; (struct pbuf *)q++)//q = q->next)  //-------------fin Nat-------------------
    {
      u16_t i;
      u16_t *ptr = (u16_t *)q->payload;
      /* Send the data from the pbuf to the interface, one pbuf at a
         time. The size of the data in each pbuf is kept in the ->len
         variable. */ 
      for(i = 0; i < q->len; i += 2)         //----------------Nat--------------------
      {                                      //Recive&Transmit data port in I/O Mode I/OModeBaseAdd+0x0000
        *pPacketPageRxTxData = *ptr++;					 //pag 75 CS8900A DataSheet
      }                                      //-------------fin Nat-------------------
#if (CS8900_STATS > 0)
      ((struct cs8900if *)netif->state)->sentbytes += q->len;      //Natalia
#endif
    }
#if (CS8900_STATS > 0)
    ((struct cs8900if *)netif->state)->sentpackets++;          //Natalia
#endif
  }

  return ERR_OK;
}
 
u16_t cs8900_inputData(void)          //Natalia
{
  u16_t uiRegNumber;
  u16_t len = 0;
  u16_t event_type;
  u16_t i;
  u16_t *ptr;  
  unsigned char ucCount;     
  u16_t irq_status = 0x0000U;   
  u16_t rx_status=0x0000U;
  unsigned int uiFrameAddress;
  
  
  //*pPacketPagePointer=CS_PP_ISQ;
  *pPacketPagePointer=CS_PP_RXEVENT;
  irq_status = *pPacketPageISQ;                     //----------------Nat--------------------
  										  //Pag 75, accede a ISQ diretamente en I/O Mode
  										  //segn packetpage architecture  
  										  //Cuando ocurre un evento q tiene la int habilitada
  										  //El contenido del registro q da los datos del evento
  										  //es mapeado en ISQ, y en ISQ tb se indica q regitro se ha mapeado
  										  //Es decir, si el contenido de ISQ!=0, entonces indica:
  										  //xxxx xxxx xxyy yyyy
  										  //xxxx xxxx xx-------->Inidica Reg content
  										  //			yy yyyy->Inidica Reg number
  										  //-------------fin Nat------------------ 
  ucCount=0;
  //while(*pPacketPageData==0){
  while(irq_status==0){	
  	*pPacketPagePointer=CS_PP_RXEVENT;
  	if((*pPacketPageData==0x0004)&(ucCount==255)){return(0);}
  	else{
  	    if(*pPacketPageData!=0x0004){rx_status=*pPacketPageData;break;}
  	    else{ucCount++;}
  	}
  	irq_status = *pPacketPageISQ; //*pPacketPagePointer=CS_PP_ISQ;
  } 
  if(irq_status!=0){
  	uiRegNumber=irq_status&0x003F;//Se toma el registro q ha producido la int
  }
  else{return(0);}
//  if((rx_status&0x0100)==0x0100){
//  	return(1);}
  //if ((*pPacketPageData & 0x003fU) == 0x0004U){    //Receiver Event
   if(uiRegNumber==0x0004){ //RxEvent, hay q tratar		
  		event_type=irq_status&0xFFC0;
  										                           
                                             
                                          //----------------Nat--------------------
    									  //Basic I/O Mode receive operation, pag 76
    									  //Repetitive Read intructions from RXTXREG
    									  //returns: RxStatus first, RxLength later
    									  //and finally RxFrameData
    									  //-------------fin Nat------------------
  // read RxStatus
 // event_type = *pPacketPageRxTxData;                   //----------------Nat--------------------
  										  //Primera lectura de RxTxDataPort devuelve Registro RxCTL, registro 5    
  										  //Recive&Transmit data port in I/O Mode I/OModeBaseAdd+0x0000
  										  //xxxx 11x1 xxxx xxxx
  										  //		   1---------->Inidica trama aceptada con correcto CRC
  										  //		11------------>Inidica trama aceptada por Individual Address o Broadcast
                                          //-------------fin Nat------------------

  //return(event_type); //para llamar ahora a la funcin q lee la trama

  // correctly received frame, either broadcast or individual address?
  // TODO: think where these checks should appear: here or in cs8900_input()
  if ((event_type & 0x0100U/*RxOK*/) && (event_type & 0x0c00U/*Broadcast | Individual*/))
  {
    event_type = 0; 
    // read RxLength
	*pPacketPagePointer=CS_PP_RXLEN;
	len=*pPacketPageData; 
	 
    //Intento leer el mensaje
	uiFrameAddress=(MEM_BASE + CS_PP_RXFRAME);
	*pPacketPagePointer=uiFrameAddress;
	if (len > 0){//Leer los datos y guardarlos en el buffer
        for(i=0;i<len;i++){  
        	*pPacketPagePointer=uiFrameAddress;
        	uiEthBuffer_Rx1[i]=*pPacketPageData;
        	*pPacketPagePointer=uiFrameAddress++;	
        }
		return(1);
	}
    else{return(0);}

  //  len = *pPacketPageRxTxData;                        //----------------Nat--------------------
  										  //Segunda lectura de RxTxDataPort devuelve longitud de la trama 
  										  //-------------fin Nat------------------ 
  // read RxStatus
 //   DEBUGF(ETH_DEBUG, ("cs8900_input: packet len %u\n", len)); //ensear en pantalla
    // positive length?
   }//fin de event type          
  }//fin de regnumber
  return(0);
}

/*
 * cs8900_input():
 *
 * Allocates a pbuf and transfers the bytes of the incoming
 * packet from the interface into the pbuf.
 *
 */

// To be called just after reading an ISQ event
// containing the "Receiver Event" register.
//
// This function copies a frame from the CS8900A.
// It is designed fail-safe:
// - It does not assume a frame is actually present.
// - It checks for non-zero length
// - It does not overflow the frame buffer
//static elimando al utilizar esta funcin como esxterna me da warning el static
struct pbuf *cs8900_input(struct netif *netif)//, u16_t event_type)          //Natalia
{
  struct pbuf *p = 0, *q;   //Natalia
  u16_t len = 0;
  u16_t event_type;
  u16_t i;
  u16_t *ptr; 
  unsigned int uiFrameAddress;                               
  u32_t irq_status;                                           
                                          //----------------Nat--------------------
    									  //Basic I/O Mode receive operation, pag 76
    									  //Repetitive Read intructions from RXTXREG
    									  //returns: RxStatus first, RxLength later
    									  //and finally RxFrameData
    									  //-------------fin Nat------------------
  // read RxStatus
  *pPacketPagePointer=CS_PP_RXEVENT;
  irq_status = *pPacketPageISQ;
  event_type=irq_status&0xFFC0;
 // event_type = *pPacketPageRxTxData;                   //----------------Nat--------------------
  										  //Primera lectura de RxTxDataPort devuelve Registro RxCTL, registro 5    
  										  //Recive&Transmit data port in I/O Mode I/OModeBaseAdd+0x0000
  										  //xxxx 11x1 xxxx xxxx
  										  //		   1---------->Inidica trama aceptada con correcto CRC
  										  //		11------------>Inidica trama aceptada por Individual Address o Broadcast
                                          //-------------fin Nat------------------

  // correctly received frame, either broadcast or individual address?
  // TODO: think where these checks should appear: here or in cs8900_input()
  if ((event_type & 0x0100U/*RxOK*/) && (event_type & 0x0c00U/*Broadcast | Individual*/))
  {
    event_type = 0; 
    *pPacketPagePointer=CS_PP_RXLEN;
	len=*pPacketPageData; 

    // read RxLength
    //len = *pPacketPageRxTxData;                        //----------------Nat--------------------
  										  //Segunda lectura de RxTxDataPort devuelve longitud de la trama 
  										  //-------------fin Nat------------------ 
  // read RxStatus
 //   DEBUGF(ETH_DEBUG, ("cs8900_input: packet len %u\n", len)); //ensear en pantalla
    // positive length?
    if (len > 0)
    {
      // allocate a pbuf chain with total length 'len'
	  p = pbuf_alloc(PBUF_LINK, len, PBUF_POOL);
   	  uiFrameAddress=(MEM_BASE+CS_PP_RXFRAME); 
	  *pPacketPagePointer=uiFrameAddress; 
      if (p != NULL)
      {
        for (q = p; q != 0; q = q->next)//Natalia
	      {
          //DEBUGF(ETH_DEBUG, ("cs8900_input: pbuf @%p len %u\n", q, q->len)); //ensear en pantalla
	        ptr = q->payload;
          // TODO: CHECK: what if q->len is odd? we don't use the last byte?
	        for (i = 0; i < (q->len + 1) / 2; i++)
      	  {
	          *pPacketPagePointer=uiFrameAddress;
		      *ptr=*pPacketPageData;
			  *pPacketPagePointer=uiFrameAddress++;
			  //*ptr = *pPacketPageRxTxData;              //----------------Nat--------------------
  										  //Tercera lectura de RxTxDataPort devuelve La TRAMA
  										  //devuelve la trama de 16bits en 16bits
  										  //-------------fin Nat------------------ 
	          ptr++;
      	  }
      	}
      }
      // could not allocate a pbuf
      else
      {
        									//----------------Nat--------------------
  										  	//Acceso al registro 3, RxCFG, receiver configuration
  										  	//Escribir en el registro para comprobar
  										  	//xxxx xxx1 x1xx xx11
  										  	//		  1---------->RxOKiE, q interrumpa si la trama se ha recibido bien, sin errores
  										  	//			 1------->Skip_1:q descarte una trama a medias cuando se recibe una nueva bien
  										  	//						es un bit q hay q ponerlo cada vez porque una vez q acta se pone a cero
  										  	//-------------fin Nat------------------ 
        
        
        // skip received frame
        // TODO: maybe do not skip the frame at this point in time?
        *pPacketPagePointer = CS_PP_RXCFG;
        *pPacketPageData = (0x0003U | 0x0100U/*RxOKiE*/ | 0x0040U/*Skip_1*/);
#if (CS8900_STATS > 0)
        ((struct cs8900if *)netif->state)->dropped++;        //Natalia, si se permite recepcin de ms de una trama a la vez
#endif
        len = 0;
      }
    }
    // length was zero
    else
    {
    }
  }
  return p;
}


// To be called when the cs8900a needs service. Does
// not assume the cs8900a needs service. Does test the
// cs8900a whether it needs service. 
//
//
// As such, may be used robustly called as a deferred
// (or "late") interrupt handler, or may be called in
// a loop to implement polling, or both.
//
// Use cs8900if_service() from your application instead
// of this function. 

static void cs8900_service(struct netif *netif)     //Natalia
{
  // amount of ISQ's to handle (> 0) in one cs8900_service() call
  unsigned char events2service = 1;
  // NOTES:
  // static, so only initialized to zero at program start.
  // irq_status will always hold the last ISQ event register that
  // still needs service. As such, we may leave this function if
  // we encounter an event we cannot service yet, and return later
  // to try to service it.
  static u16_t irq_status = 0x0000U;

  // The "cs8900_needs_service" flag indicates whether any events
  // still need to be serviced.
  // clear flag here. 
  // a receive interrupt can, *concurrently with this function*,
  // set this flag on new ISQ event occurences.
  // we will re-evaluate the correct setting of this flag at
  // function exit (below).
  ((struct cs8900if *)netif->state)->needs_service = 0;       //Natalia
  //leds_off(LED_FP0);//Natalia

  // no unhandled irq_status left?
  if (irq_status == 0x0000U)
  {
    // read ISQ register
    irq_status = *pPacketPageISQ;                     //----------------Nat--------------------
  										  //Pag 75, accede a ISQ diretamente en I/O Mode
  										  //segn packetpage architecture  
  										  //Cuando ocurre un evento q tiene la int habilitada
  										  //El contenido del registro q da los datos del evento
  										  //es mapeado en ISQ, y en ISQ tb se indica q regitro se ha mapeado
  										  //Es decir, si el contenido de ISQ!=0, entonces indica:
  										  //xxxx xxxx xxyy yyyy
  										  //xxxx xxxx xx-------->Inidica Reg content
  										  //			yy yyyy->Inidica Reg number
  										  //-------------fin Nat------------------ 
  }
  // ISQ interrupt event, and allowed to service in this loop?
  while ((irq_status != 0x0000U) && (events2service-- > 0))
  {                                       
  										  //----------------Nat-------------------- 
  										  //Ver q evento produce la interrupcin
  										  //Ver el valor de RegNumber
  										  //si es=Register4= receiver Event
  										  //-------------fin Nat------------------     
  										  
    // investigate event 
    //-----------------------------------Receive Event--------------------------------------------
    if ((irq_status & 0x003fU) == 0x0004U)    //Receiver Event
    {
      // correctly received frame, either broadcast or individual address
      // TODO: think where these checks should appear: here or in cs8900_input()
      if ((irq_status & 0x0100U) && (irq_status & 0x0c00U)) //RxOK,Broadcast | Individual
      {
        // read the frame from the cs8900a                //----------------Nat-------------------- 
        cs8900if_input(netif);                            //Ver los valores de RxEvent, register4
													      //xxxx 11x1 xxxx xxxx
													      //	 	1----------->Indica recepcin de trama con CRC Ok
													      //     11------------->Indica recepcin de trama por Inidividual Address o Broadcast
													      //-------------fin Nat------------------
      }                                                   
      else                                                
      {                                                   
        // skip this frame          					 //----------------Nat--------------------                         
        *pPacketPagePointer = CS_PP_RXCFG;                          //Si no se  ha cumplido lo de antes, no se ha recibido bien
        *pPacketPageData |= 0x0040U;					 //entonces se pone en RxCFG, register 3
        												 //xxxx xxxx x1xx xxxx
        												 //			  1------->Skip_1, se pone a uno para descartar la trama
        												 //						 q no se haya recibido bien o no sea para nosotros(address)
        												 //-------------fin Nat------------------
#if (CS8900_STATS > 0)
        ((struct cs8900if *)netif->state)->dropped++;          //Natalia
#endif
      }
    }
#if (CS8900_STATS > 0)                                                       
														//---------------Nat------------------
    //-----------------------------------RxMiss Event--------------------------------------------
    else if ((irq_status & 0x003fU) == 0x0010U)         //Int q salta para decir q hay q leer RxMissCount antes de q se produzca overflow
    {																	//RxMissCount, tramas perdidas debido al tamao del buffer
  	  ((struct cs8900if *)netif->state)->missed += (irq_status >> 6);   //Natalia
  	}
    else if ((irq_status & 0x003fU) == 0x0012U)          //Txcol Event:Transmit collision counter, collision en la transmisin
    {																	//Salta la int antes de q se produzca overflow en el contador
  	  ((struct cs8900if *)netif->state)->collisions += (irq_status >> 6); //Natalia
  	}
#endif
    // read ISQ register                                
    irq_status = *pPacketPageISQ;        							//Leer de nuevo, ISQ por si se producen ms eventos
  }                                                     

  // we did not deplete the ISQ?
  if (irq_status != 0x0000U)         					//ISQ=0, ya no hay ms eventos en cola q atender
  {
    // the cs8900a still needs service
    ((struct cs8900if *)netif->state)->needs_service = 1;   //Natalia
   // leds_on(LED_FP0);    //Natalia
  }
#if (CS8900_STATS > 1)
  // read RxMiss Counter (zeroes itself upon read)       //Se ponen RxMisCountery TxCollConunter a cero una vez ledos
  *pPacketPagePointer = CS_PP_RXMISS;
  ((struct cs8900if *)netif->state)->missed += (*pPacketPageData >> 6);    //Natalia
  // read RxCol Counter (zeroes itself upon read)
  *pPacketPagePointer = CS_PP_TXCOL;
  ((struct cs8900if *)netif->state)->collisions += (*pPacketPageData >> 6);   //Natalia
#endif                                                   //----------------fin Nat-----------------
}

      

void cs8900if_service(struct netif *netif)
{                                                           //Natalia
  // is there a reason to call the service routine?
  if (((( struct cs8900if *)netif->state)->needs_service) || (((struct cs8900if *)netif->state)->use_polling))   //Natalia
  {
    cs8900_service(netif);
  }
} 



/*-----------------------------------------------------------------------------------*/
/*
 * cs8900if_output():
 *
 * This function is called by the TCP/IP stack when an IP packet
 * should be sent. After ARP address lookup, it calls cs8900_output() to
 * do the actual transmission of the packet.
 *
 */
/*-----------------------------------------------------------------------------------*/

static err_t
cs8900if_output(struct netif *netif, struct pbuf *p,
		   struct ip_addr *ipaddr)                 //Natalia
{
  struct cs8900if *cs8900if = netif->state;    //Natalia
  struct pbuf *q = NULL;                       //Natalia
  struct eth_hdr *ethhdr;                      //Natalia
  struct eth_addr *dest, mcastaddr;            //Natalia
  struct ip_addr *queryaddr;                   //Natalia
  err_t err;
  u8_t i;
  // Make room for Ethernet header. 
     if(pbuf_header(p, 14) != 0) {
     
    // The pbuf_header() call shouldn't fail, but we allocate an extra
//       pbuf just in case. 
    q = pbuf_alloc(PBUF_LINK, 14, PBUF_RAM);
	if(q == NULL) {
      return ERR_MEM;
    }
    pbuf_chain(q, p);
    p = q;
  }

  // Construct Ethernet header. Start with looking up deciding which
  //   MAC address to use as a destination address. Broadcasts and
  //   multicasts are special, all other addresses are looked up in the
  //   ARP table. 
  queryaddr = ipaddr;
 	if(ip_addr_isbroadcast(ipaddr, &(netif->netmask))) {
    dest = (struct eth_addr *)&ethbroadcast;         //Natalia
  }
   else if(ip_addr_ismulticast(ipaddr)) {
    // Handle multicast MAC address hashing. 
    mcastaddr.addr[0] = 0x01;
    mcastaddr.addr[1] = 0x00;
    mcastaddr.addr[2] = 0x5e;
    mcastaddr.addr[3] = ip4_addr2(ipaddr) & 0x7f;
    mcastaddr.addr[4] = ip4_addr3(ipaddr);
    mcastaddr.addr[5] = ip4_addr4(ipaddr);
    dest = &mcastaddr;
  } else {

   if(ip_addr_maskcmp(ipaddr, &(netif->ip_addr), &(netif->netmask))) {
      // Use destination IP address if the destination is on the same
      //   subnet as we are. 
      queryaddr = ipaddr;
    } else {
      // Otherwise we use the default router as the address to send
      //   the Ethernet frame to. 
      queryaddr = &(netif->gw);
    }
    dest = arp_lookup(queryaddr);
  }

  // If the arp_lookup() didn't find an address, we send out an ARP
  //   query for the IP address. 
  if(dest == NULL) {
    struct pbuf *r;              //Natalia
     
    r = arp_query(netif,((struct cs8900if *)netif->state)->ethaddr, queryaddr);  //Natalia   (cs8900if*)
	if(r != NULL) {
      err = cs8900_output(netif, r);
#if defined(CS8900_TX_QUEUE) && (CS8900_TX_QUEUE > 0)
      // a rather stupid attempt to queue packets during ARP resolution
      // arp request sent?
      if (err == ERR_OK)
			{
			  // enqueue packet 
        if (((struct cs8900if *)netif->state)->tx_queue == NULL)        //Natalia
          ((struct cs8900if *)netif->state)->tx_queue = p;            //Natalia
				return ERR_OK;
			}
#else
	pbuf_free(r);
    return err;
#endif
    }
    return ERR_MEM;
  }
  ethhdr = p->payload;
  for(i = 0; i <  6; i++) {
    ethhdr->dest.addr[i] = dest->addr[i];
    ethhdr->src.addr[i] = ((struct cs8900if *)netif->state)->ethaddr->addr[i];      //Natalia
  }
  ethhdr->type = htons(ETHTYPE_IP);

  err = cs8900_output(netif, p);
  // we allocated the ethernet header ourselves
  // dechain it from the original pbuf
  // then free it
  if (q != NULL)
  {
    pbuf_dechain(q);
    pbuf_free(q);
  }
  return err;
}                                

/*-----------------------------------------------------------------------------------*/
/*
 * cs8900if_input():
 *
 * This function should be called when a packet is ready to be read
 * from the interface. It uses the function cs8900_input() that
 * should handle the actual reception of bytes from the network
 * interface.
 *
 */
/*-----------------------------------------------------------------------------------*/

static void
cs8900if_input(struct netif *netif)              //Natalia
{
  struct cs8900if *cs8900if = netif->state;      //Natalia
  struct eth_hdr *ethhdr;                        //Natalia
  struct pbuf *p;                                //Natalia

  p = cs8900_input(netif);

  if(p == NULL) {
    return;
  }
  ethhdr = p->payload;

  switch(htons(ethhdr->type)) {
  case ETHTYPE_IP:
    arp_ip_input(netif, p);
	pbuf_header(p, -14);
	netif->input(p,(struct netif*) netif);
    break;
  case ETHTYPE_ARP:
    p = arp_arp_input(netif, ((struct cs8900if *)netif->state)->ethaddr, p);    //Natalia
	if(p != NULL) {
      cs8900_output(netif, p);
      pbuf_free(p);
	}
#if defined(CS8900_TX_QUEUE) && (CS8900_TX_QUEUE > 0)
    // this could have been an ARP reply
    // possible resolving a queued packet destination address
    else
		{
		  if (((struct cs8900if *)netif->state)->tx_queue)        //Natalia
			{
        struct eth_addr *dest;              //Natalia
        struct ip_hdr *iphdr;               //Natalia
        // point to IP header												 			 
		pbuf_header(((struct cs8900if *)netif->state)->tx_queue, -14);        //Natalia
				iphdr = ((struct cs8900if *)netif->state)->tx_queue->payload; //Natalia
        // point to link header						
		pbuf_header((( struct cs8900if *)netif->state)->tx_queue, 14);        //Natalia
				// address can now be resolved?
		if ((dest = arp_lookup(&(iphdr->dest))))
		{
				  char i;
          ethhdr = ((struct cs8900if *)netif->state)->tx_queue->payload;     //Natalia
          for(i = 0; i < 6; i++)
          {
            ethhdr->dest.addr[i] = dest->addr[i];
            ethhdr->src.addr[i] = ((struct cs8900if *)netif->state)->ethaddr->addr[i];   //Natalia
					}
          ethhdr->type = htons(ETHTYPE_IP);
          cs8900_output(netif, ((struct cs8900if *)netif->state)->tx_queue);     //Natalia
        }
				((struct cs8900if *)netif->state)->tx_queue = NULL;          //Natalia
			}
		}
#endif
    break;
  default:
  	pbuf_free(p);
	break;
  }
} 

/*-----------------------------------------------------------------------------------*/
/*
 * cs8900if_init():
 *
 * Should be called at the beginning of the program to set up the
 * network interface. It calls the function low_level_init() to do the
 * actual setup of the hardware.
 *
 */
/*-----------------------------------------------------------------------------------*/
void
cs8900if_init(struct netif *netif)
{
  struct cs8900if *cs8900if;          //Natalia

  // TODO: check result!
  //99//99//99 Solo para poder compilar Natalia
  cs8900if =malloc(sizeof(cs8900if)); 
 //99//99//99 Natalia utilizar malloc de la libreria de hitachi
  
  //cs8900if = mem_malloc(sizeof(cs8900if));      //Natalia, hay q cambiar...
  //99//99//99 Solo para poder compilar Natalia
  if (cs8900if == NULL) return;


  netif->state = cs8900if;
  netif->name[0] = IFNAME0;
  netif->name[1] = IFNAME1;
  netif->output = cs8900if_output;   
  netif->linkoutput = cs8900_output; 

  cs8900if->ethaddr = (struct eth_addr *)&(netif->hwaddr[0]);          //Natalia
  // initially assume no ISQ event
  cs8900if->needs_service = 0;
  // set to 1 if polling method is used
  cs8900if->use_polling = 0;

#if (CS8900_STATS > 0)
  // number of interrupts (vector calls)
  cs8900if->interrupts = 0;
  cs8900if->missed = 0;
  cs8900if->dropped = 0;
  cs8900if->sentpackets = 0;
  cs8900if->sentbytes = 0;
#endif

  cs8900_init(netif); 

  // TODO: remove this hack
  //cs8900if_netif = netif;

  arp_init();
}

/*-----------------------------------------------------------------------------------*/

// this basically dumps a array of bytes, length "len" into a UDP message. It bypasses
// any functionality within lwIP, acting as a low level debug output function (in case
// your hardware has no, or limited, display output available) to debug lwIP itself. 
//
// The checksum routines are not implemented yet here, so do not expect your target's
// IP stack to accept this packet. It works great, however, in combination with a
//  packet snooper, such as Ethereal (www.ethereal.org).
//
void cs8900_send_debug(unsigned char *p, unsigned int len)
{
	int tries = 0, i;
                                                   
												//--------------------Nat-------------------
												//Read Register 14, Line Status 
												//Antes de enviar comprueba q LinkOK
												//xxxx xxxx 1x01 0100
												//			1-------->LinkOK
												//			  01 0100->fijos
												//-------------------fin Nat-------------------

  // exit if link has failed
  *pPacketPagePointer = CS_PP_LINESTATUS;
  if ((*pPacketPageData & 0x0080U/*LinkOK*/) == 0) return; // TODO: find a correct error code
  
												//--------------------Nat-------------------   
												//Transmit command request TxCMD, write only
												//0000 0000 1100 1001
												//			11------->start transmision after the entire frame is in Cs8900A
												//            00 1001->fijos
												//-------------------fin Nat-------------------
  /* transmit command */                                                            
  *pPacketPageTxCMD = 0x00C9U;
  *pPacketPageLength = (14 + 20 + 8 + len < 60)?60:(14 + 20 + 8 + len);       												
  												//--------------------Nat-------------------
  												//En I/O Mode, packet page architecture, pag 75
  												//la long de la trama a enviar se coloca directamente en TxLength
												//-------------------fin Nat-------------------
                                                

												//--------------------Nat-------------------  
												//Leer el registro BusStatus,register 18
												// xxxx xxx1 xxxx xxxx
												//		   1---------->indica q el chip CS8900A 
												//						est preparado para recibir una trama del micro
												//-------------------fin Nat-------------------
  *pPacketPagePointer = CS_PP_BUSSTATUS;
  // not ready for transmission and still within 100 retries?
  while(((*pPacketPageData & 0x0100U/*Rdy4TxNOW*/) == 0) && (tries++ < 100))
  {                                                  
  
  												//--------------------Nat------------------- 
  												//Leer el registro de recepcin para configurarlo de manera q descarte
  												//la recepcin de tramas
  												//Receiver configuration, RxCFG, register 3
  												// 0000 0001 0100 0011
  												//		   1------------>RxOKiE, interrupt if fram received OK
  												//			  1--------->Skip_1,when receiving a new fram skip a frame in the buffer
  												//			   00 0011-->fijos
												//-------------------fin Nat-------------------
    // throw away the last committed received frame
    *pPacketPagePointer = CS_PP_RXCFG;
    *pPacketPageData = (0x0003U | 0x0040U/*Skip_1*/ | 0x0100U/*RxOKiE*/);
    *pPacketPagePointer = CS_PP_BUSSTATUS;                 //--------------------Nat-------------------  
												//Leer el registro BusStatus,register 18  
												//-------------------fin Nat-------------------
    /* cs8900if->dropped++; CHECK: we do not know if we actually will drop a frame here, do we? */ 
  }
  												//--------------------Nat-------------------  
  												//El fit Rdy4TxNow=1, bit 8 del registro BusStatus,register 18  
  												//el chip est preparado para enviar la trama 
  												//se va colocando en RxTxReg(pag 75),los bytes a enviar, de 16 en 16bits
												//-------------------fin Nat-------------------
  // ready to transmit?                          
  if((*pPacketPageData & 0x0100U/*Rdy4TxNOW*/) != 0)
  {
    u16_t data, checksum = 0;
    u32_t udp_checksum = 0;
	
	// destination Ethernet address
    *pPacketPageRxTxData = 0x00B0;//0xffff;//htons((0x00 << 8) | 0x0a); 
    *pPacketPageRxTxData = 0xD05E;//0xffff;//htons((0x24 << 8) | 0xc5); 
    *pPacketPageRxTxData = 0x3497;//0xffff;//htons((0x72 << 8) | 0x6d); 
    // source Ethernet address
    *pPacketPageRxTxData = htons(((u16_t)ETHADDR0 << 8U) | (u16_t)ETHADDR1); 
    *pPacketPageRxTxData = htons(((u16_t)ETHADDR2 << 8U) | (u16_t)ETHADDR3); 
    *pPacketPageRxTxData = htons(((u16_t)ETHADDR4 << 8U) | (u16_t)ETHADDR5);
    // frame type
    *pPacketPageRxTxData = htons(0x0800);
    // TOS, version
    *pPacketPageRxTxData = htons(((0x40 | 0x05) << 8) | 0x00); 
    // length
    *pPacketPageRxTxData = htons(data = 20 + 8 + len); //htons(20 + 12 + len);  cambiado por cs8900 new driver
	checksum += data;
    // identifier
    *pPacketPageRxTxData = htons(data=0); 
	checksum += data;
    // fragment offset
    *pPacketPageRxTxData = htons(data=0); 
	checksum += data;
    // TTL, UDP protocol
    *pPacketPageRxTxData = htons(data=(255U << 8) | 17U); 
	checksum += data;
    // checksum
    //*pPacketPageRxTxData = htons(0);  //Cuando no se calculaba el checksum
	checksum += htons(0xc0a8);
    checksum += htons(0xdede);
    checksum += 0xc0a8U;
    checksum += 0x0001U;
    checksum += 6; // LW: kludge/hack: checksum calculation seems to be wrong somehow
    // LW: this seems (?) to fix it
    // checksum
    *pPacketPageRxTxData = htons(~checksum); 


    // source IP   192.168.222.222
    *pPacketPageRxTxData = htons(0xc0a8); //htons((htonl(cs8900if_netif->ip_addr.addr) & 0xffff0000U) >> 16); 
    // source IP
    *pPacketPageRxTxData = htons(0xdede);//htons(htonl(cs8900if_netif->ip_addr.addr) & 0x0000ffffU); 
    // destination IP  192.168.2.211
    *pPacketPageRxTxData = htons(0xc0a8); 
    // destination IP
    *pPacketPageRxTxData = htons(0x02d3);//htons(0x0001); 
    // source port 3000
    *pPacketPageRxTxData = htons(9999U); 
    // destination port 3000
    *pPacketPageRxTxData = htons(8888U); 
    // UDP length
    *pPacketPageRxTxData = htons(len); 
    // UDP checksum
    //*pPacketPageRxTxData = htons(0);
	udp_checksum =  0xc0a8U;
    udp_checksum += 0xdedeU;
    udp_checksum += 0xc0a8U;
    udp_checksum += 0x0001U;
    udp_checksum += 0x0011U;
    udp_checksum += (8 + len);
    udp_checksum += 9999;
    udp_checksum += 8888;
    udp_checksum += (8 + len);
    udp_checksum += cs8900_chksum(p, len);
    while(udp_checksum >> 16) {
      udp_checksum = (udp_checksum & 0xffffUL) + (udp_checksum >> 16);
    }    

    *pPacketPageRxTxData = htons(~(udp_checksum & 0xffff));
	  // UDP data
    

    for (i = 0; i < len; i += 2)
    {
      *pPacketPageRxTxData = htons((p[i] << 8) | p[i + 1]);
    } 
		while (i < 60)
		{
      *pPacketPageRxTxData = 0;
			i += 2;
		}
  }                									//Trama enviada---------fin Nat.
}
static u32_t cs8900_chksum(void *dataptr, int len)
{
  u32_t acc = 0;
  u16_t *ptr = (u16_t *)dataptr;
    
  for(acc = 0; len > 1; len -= 2) {
    acc += *ptr;
    ptr++;
  }
  /* add up any odd byte */
  if(len == 1) {
    acc += htons((u16_t)((*(u8_t *)ptr) & 0xffU) << 8);
  }
  return acc;
}
